# ***** BEGIN LICENSE BLOCK *****
# Version: RCSL 1.0/RPSL 1.0/GPL 2.0
#
# Portions Copyright (c) 1995-2002 RealNetworks, Inc. All Rights Reserved.
# Portions Copyright (c) 2004 Robert Kaye. All Rights Reserved.
#
# The contents of this file, and the files included with this file, are
# subject to the current version of the RealNetworks Public Source License
# Version 1.0 (the "RPSL") available at
# http://www.helixcommunity.org/content/rpsl unless you have licensed
# the file under the RealNetworks Community Source License Version 1.0
# (the "RCSL") available at http://www.helixcommunity.org/content/rcsl,
# in which case the RCSL will apply. You may also obtain the license terms
# directly from RealNetworks.  You may not use this file except in
# compliance with the RPSL or, if you have a valid RCSL with RealNetworks
# applicable to this file, the RCSL.  Please see the applicable RPSL or
# RCSL for the rights, obligations and limitations governing use of the
# contents of the file.
#
# This file is part of the Helix DNA Technology. RealNetworks is the
# developer of the Original Code and owns the copyrights in the portions
# it created.
#
# This file, and the files included with this file, is distributed and made
# available on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
# EXPRESS OR IMPLIED, AND REALNETWORKS HEREBY DISCLAIMS ALL SUCH WARRANTIES,
# INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
#
# Technology Compatibility Kit Test Suite(s) Location:
#    http://www.helixcommunity.org/content/tck
#
# --------------------------------------------------------------------
#
# picard is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# picard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with picard; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Contributor(s):
#   Robert Kaye
#
#
# ***** END LICENSE BLOCK *****

import sys, urllib, webbrowser, os, wx
from tunepimp import tunepimp, metadata
from tunepimp import track as tptrack
from lib import wpath
import launch

class FileLookup(launch.Launch):
    
    def __init__(self, parent, server, port, localPort):
        launch.Launch.__init__(self, parent)
        self.server = server
        self.localPort = localPort
        self.port = port

    def trackLookup(self, trackId):
        return self.launch("http://" + self.server + ":" + str(self.port) + "/showtrack.html?mbid=" + 
                        str(trackId) + "&tport=" + str(self.localPort))

    def albumLookup(self, albumId):
        return self.launch("http://" + self.server + ":" + str(self.port) + "/showalbum.html?mbid=" + 
                        str(albumId) + "&tport=" + str(self.localPort))

    def artistLookup(self, artistId):
        return self.launch("http://" + self.server + ":" + str(self.port) + "/showartist.html?mbid=" + 
                        str(artistId) + "&tport=" + str(self.localPort))

    def artistSearch(self, query):
        assert(query.__class__.__name__ == 'unicode')
        query = urllib.quote(query.encode('utf8', 'replace'))
        return self.launch("http://" + self.server + ":" + str(self.port) + "/newsearch.html?limit=25" + 
                        "&table=artist&search=" + query + "&tport=" + str(self.localPort))

    def albumSearch(self, query):
        assert(query.__class__.__name__ == 'unicode')
        query = urllib.quote(query.encode('utf8', 'replace'))
        return self.launch("http://" + self.server + ":" + str(self.port) + "/newsearch.html?limit=25" + 
                        "&table=album&search=" + query + "&tport=" + str(self.localPort))

    def trackSearch(self, query):
        assert(query.__class__.__name__ == 'unicode')
        query = urllib.quote(query.encode('utf8', 'replace'))
        return self.launch("http://" + self.server + ":" + str(self.port) + "/newsearch.html?limit=25" + 
                        "&table=track&search=" + query + "&tport=" + str(self.localPort))

    def tagLookup(self, ar, al, tr, trackNum, duration, fileName, artistId, trackId, trmId):
        assert(ar.__class__.__name__ == 'unicode')
        assert(al.__class__.__name__ == 'unicode')
        assert(tr.__class__.__name__ == 'unicode')
        al = urllib.quote(al.encode('utf8', 'replace'))
        ar = urllib.quote(ar.encode('utf8', 'replace'))
        tr = urllib.quote(tr.encode('utf8', 'replace'))
        fileName = urllib.quote(wpath.wpath().basename(fileName).encode('utf-8', 'replace'))
        artistId = urllib.quote(artistId.encode('utf-8', 'replace'))
        trackId = urllib.quote(trackId.encode('utf-8', 'replace'))
        trmId = urllib.quote(trmId.encode('utf-8', 'replace'))
        url = "http://" + self.server + ":" + str(self.port) + "/taglookup.html" \
                        "?tport=" + str(self.localPort) + \
                        "&artist=" + ar.encode('UTF-8', 'replace') + \
                        "&album=" + al.encode('UTF-8', 'replace') + \
                        "&track=" + tr.encode('UTF-8', 'replace') + \
                        "&tracknum=" + str(trackNum) + \
                        "&duration=" + str(duration) + \
                        "&filename=" + fileName + \
                        "&trackid=" + trackId + \
                        "&artistid=" + artistId + \
                        "&trmid=" + trmId.encode('UTF-8', 'replace') 
        return self.launch(url)
