# ***** BEGIN LICENSE BLOCK *****
# Version: RCSL 1.0/RPSL 1.0/GPL 2.0
#
# Portions Copyright (c) 1995-2002 RealNetworks, Inc. All Rights Reserved.
# Portions Copyright (c) 2004 Robert Kaye. All Rights Reserved.
#
# The contents of this file, and the files included with this file, are
# subject to the current version of the RealNetworks Public Source License
# Version 1.0 (the "RPSL") available at
# http://www.helixcommunity.org/content/rpsl unless you have licensed
# the file under the RealNetworks Community Source License Version 1.0
# (the "RCSL") available at http://www.helixcommunity.org/content/rcsl,
# in which case the RCSL will apply. You may also obtain the license terms
# directly from RealNetworks.  You may not use this file except in
# compliance with the RPSL or, if you have a valid RCSL with RealNetworks
# applicable to this file, the RCSL.  Please see the applicable RPSL or
# RCSL for the rights, obligations and limitations governing use of the
# contents of the file.
#
# This file is part of the Helix DNA Technology. RealNetworks is the
# developer of the Original Code and owns the copyrights in the portions
# it created.
#
# This file, and the files included with this file, is distributed and made
# available on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
# EXPRESS OR IMPLIED, AND REALNETWORKS HEREBY DISCLAIMS ALL SUCH WARRANTIES,
# INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
#
# Technology Compatibility Kit Test Suite(s) Location:
#    http://www.helixcommunity.org/content/tck
#
# --------------------------------------------------------------------
#
# picard is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# picard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with picard; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Contributor(s):
#   Robert Kaye
#
#
# ***** END LICENSE BLOCK *****

import wx, events
import sys,os
import dataobjs, artist, track, util
import musicbrainz
from tunepimp import tunepimp, metadata
from tunepimp import track as tptrack
import string

NON_ALBUM_TRACKS = "[non-album tracks]"

class AlbumLoadError(LookupError):
    "This is used to indicate that a given album was not found on the server"
   
class AlbumUnmatched(dataobjs.DataObject):
    '''The AlbumUnmatched class is a data object used in the album panel tree to identify
       the tree node'''
    def __init__(self, config, id):
        dataobjs.DataObject.__init__(self, config, "", id)

class Album(dataobjs.DataObject):
    '''The Album class is a representation of an album that contains metadata for tracks and
       the list of tunepimp file ids that have been associated with a track in this album.'''
   
    def __init__(self, parent, config):
        dataobjs.DataObject.__init__(self, config, "", "")
        self.parent = parent
        self.tracks = []			# track objects
        self.trackIds = {}		# mbid => index into self.tracks
        self.unmatchedFiles = []		# TP file IDs
        self.unsavedFiles = []		# TP file IDs
        self.TPFileIdToTrackSeq = {}	# TP file ID => index into self.tracks, or None
        self.duration = 0
        self.asin = u''
        self.artist = artist.Artist(config, u'', u'', u'')
        self.releaseYear = 0
        self.releaseDate = u''
        self.releaseType = u''
        self.releaseStatus = u''
        self.releaseCountry = u''
        self.numLinked = 0
        self.loaded = False
        self.hasLoadError = False

        # For server 'affinity' -- see setServerAffinity
        self.affServer = u''
        self.affPort = u''

    def dumpState(self, writer):
        writer(u"album object: %s" % ( repr(self) ))
        writer = writer.nest()
        writer(u"id: %s" % ( repr(self.id) ))
        writer(u"name: %s" % ( repr(self.name) ))
        writer(u"trackIds: %s" % ( repr(self.trackIds) ))
        writer(u"unmatchedFiles: %s" % ( repr(self.unmatchedFiles) ))
        writer(u"duration: %s" % ( repr(self.duration) ))
        writer(u"asin: %s" % ( repr(self.asin) ))
        writer(u"artist: %s" % ( repr(self.artist) ))
        self.artist.dumpState(writer)
        writer(u"releaseYear: %s" % ( repr(self.releaseYear) ))
        writer(u"releaseDate: %s" % ( repr(self.releaseDate) ))
        writer(u"releaseType: %s" % ( repr(self.releaseType) ))
        writer(u"releaseStatus: %s" % ( repr(self.releaseStatus) ))
        writer(u"releaseCountry: %s" % ( repr(self.releaseCountry) ))
        writer(u"numLinked: %s" % ( repr(self.numLinked) ))
        writer(u"loaded: %s" % ( repr(self.loaded) ))
        if len(self.tracks):
            writer(u"track count: %d" % ( len(self.tracks) ))
            for i in xrange(0, len(self.tracks)):
                t = self.tracks[i]
                t.dumpState(writer)
        else:
            writer(u"this album has no tracks")

    def setArtist(self, ar):
        self.artist = ar
       
    def getArtist(self):
        return self.artist

    def getNumTracks(self):
        '''Return the number of tracks in this album.'''
        return len(self.tracks)

    def getTrackIds(self):
        return self.trackIds.keys()

    def getTrack(self, arg):
        '''Get a specific track from the album. index is the 0 based track number of the track to get.'''
        if arg.__class__.__name__ == "int":
            try:
                return self.tracks[arg]
            except LookupError:
                return None
        else:
            try:
                return self.tracks[self.trackIds[arg]]
            except LookupError:
                return None

    def getTrackFromFileId(self, fileId):
        seq = self.TPFileIdToTrackSeq[fileId]
        if seq is None: return None
        return self.tracks[seq]

    def findSeqOfTrack(self, trackid):
        for i in xrange(0, len(self.tracks)):
            if self.tracks[i].getId() == trackid: return i
        return None

    def isSpecial(self):
        return self.id == u"" or self.id.startswith(u"[")

    def isLoaded(self):
        return self.loaded

    def hasError(self):
        return self.hasLoadError

    def setVariousArtists(self, isVA):
        self.isVA = isVA

    def isVariousArtists(self):
        return self.isVA

    def getArtist(self):
        return self.artist

    def getAmazonAsin(self):
        return self.asin

    def setAmazonAsin(self, asin):
        self.asin = asin

    def getDuration(self):
        return self.duration

    def getDurationString(self):
        if self.duration >= 0:
            ret = u"%d:%02d" % ((self.duration / 60000), ((self.duration % 60000) / 1000))
        else:
            ret = u""

        return ret

    def setDuration(self, duration):
        self.duration = duration

    def getReleaseYear(self):
        return self.releaseYear

    def getReleaseDate(self):
        return self.releaseDate

    def getReleaseMonth(self):
        date = string.split(self.releaseDate, '-')
        return len(date) >= 2 and int(date[1]) or 0

    def getReleaseDay(self):
        date = string.split(self.releaseDate, '-')
        return len(date) >= 3 and int(date[2]) or 0

    def getReleaseCountry(self):
        return self.releaseCountry

    def getReleaseStatus(self):
        return self.releaseStatus

    def getReleaseType(self):
        return self.releaseType

    def getNumUnsavedTracks(self):
        return len(self.unsavedFiles)

    def getNumLinkedTracks(self):
        return self.numLinked

    def getUnmatchedFiles(self):
        return list(self.unmatchedFiles)

    def getNumUnmatchedFiles(self):
        return len(self.unmatchedFiles)

    def addLinkedFile(self, tpfile, seq):
        fileId = tpfile.getFileId()
        assert(fileId >= 0)
        assert(fileId not in self.TPFileIdToTrackSeq)

        track = self.tracks[seq]
        if track.isLinked():
            othertpfile = self.parent.tpmanager.findFile(track.getFileId())
            self.removeFile(othertpfile)
            self.addUnlinkedFile(othertpfile)

	    assert(not track.isLinked())

        self.TPFileIdToTrackSeq[fileId] = seq
        self.numLinked = self.numLinked + 1
        track.link(fileId)
        self.checkTrackUnsaved(tpfile)

        wx.PostEvent(self.parent, events.AlbumTrackCountsChangedEvent(self.getId()))
        wx.PostEvent(self.parent, events.AlbumFilesAddedEvent(self.getId(), [fileId]))
        wx.WakeUpIdle()

    def addUnlinkedFile(self, tpfile):
        fileId = tpfile.getFileId()
        assert(fileId >= 0)

        assert(fileId not in self.TPFileIdToTrackSeq)
        self.TPFileIdToTrackSeq[fileId] = None
        self.unmatchedFiles.append(fileId)

        wx.PostEvent(self.parent, events.AlbumTrackCountsChangedEvent(self.getId()))
        wx.PostEvent(self.parent, events.AlbumFilesAddedEvent(self.getId(), [fileId]))
        wx.WakeUpIdle()

    def removeFile(self, tpfile):
        fileId = tpfile.getFileId()
        assert(fileId >= 0)
        assert(fileId in self.TPFileIdToTrackSeq)

        seq = self.TPFileIdToTrackSeq[fileId]
        del self.TPFileIdToTrackSeq[fileId]

        if seq is None:
            self.unmatchedFiles.remove(fileId)
            wx.PostEvent(self.parent, events.AlbumTrackCountsChangedEvent(self.getId()))
            assert(fileId not in self.unsavedFiles)
        else:
            track = self.tracks[seq]
            track.unlink()
            self.numLinked = self.numLinked - 1
            if fileId in self.unsavedFiles:
                self.unsavedFiles.remove(fileId)

        wx.PostEvent(self.parent, events.AlbumTrackCountsChangedEvent(self.getId()))
        wx.PostEvent(self.parent, events.AlbumFilesRemovedEvent(self.getId(), [fileId]))
        wx.WakeUpIdle()

    def removeAllFilesFromSystem(self):
        fileIds = self.TPFileIdToTrackSeq.keys()
        for fileId in fileIds:
            tpfile = self.parent.tpmanager.findFile(fileId)
            tpfile.removeFromSystem()

    def checkTrackUnsaved(self, tpfile):
        fileId = tpfile.getFileId()
        assert(fileId >= 0)
        assert(self.TPFileIdToTrackSeq[fileId] is not None)

        tr = tpfile.getTrack()
        hasChanged = tr.hasChanged()
        tpfile.releaseTrack(tr)

        if hasChanged:
            if fileId not in self.unsavedFiles:
                self.unsavedFiles.append(fileId)
                wx.PostEvent(self.parent, events.AlbumTrackCountsChangedEvent(self.getId()))
                wx.WakeUpIdle()
        else:
            if fileId in self.unsavedFiles:
                self.unsavedFiles.remove(fileId)
                wx.PostEvent(self.parent, events.AlbumTrackCountsChangedEvent(self.getId()))
                wx.WakeUpIdle()

    def load(self):
        if not self.id:
            raise ValueError, "No album id given."

        server = self.affServer
        port = self.affPort
        if not server:
            server = self.config.settingServer
            port = self.config.settingServerPort

        self.hasLoadError = False
        mb = musicbrainz.mb()
        mb.SetServer(server.encode('UTF-8', 'replace'), self.config.settingServerPort);
        mb.SetDepth(4)
        mb.SetDebug(0)

        mb.QueryWithArgs(musicbrainz.MBQ_GetAlbumById, [self.id])
        if mb.GetResultInt(musicbrainz.MBE_GetNumAlbums) == 0:
            self.hasLoadError = True
            raise AlbumLoadError, u"The album " + self.id + u" was not found on the server. "

        mb.Select1(musicbrainz.MBS_SelectAlbum, 1)
        r = mb.GetResultData(musicbrainz.MBE_AlbumGetAlbumName)
        self.name = mb.GetResultData(musicbrainz.MBE_AlbumGetAlbumName)
        numTracks = mb.GetResultInt(musicbrainz.MBE_AlbumGetNumTracks)

        self.isVA = 0
        for i in xrange(1, numTracks + 1):
            try:
                artistId = mb.GetResultData1(musicbrainz.MBE_AlbumGetArtistId, i)
            except musicbrainz.MusicBrainzError:
                artistId = ''

            if i == 1:
                last = artistId

            if last != artistId:
                self.isVA = 1
                break

        artistId = mb.GetIDFromURL(mb.GetResultData(musicbrainz.MBE_AlbumGetAlbumArtistId))
        if artistId == musicbrainz.MBI_VARIOUS_ARTIST_ID and self.config.settingVA:
            artistName = self.config.settingVA
            sortName = self.config.settingVA
        else:
            artistName = mb.GetResultData(musicbrainz.MBE_TrackGetArtistName)
            sortName = mb.GetResultData(musicbrainz.MBE_TrackGetArtistSortName)

        self.artist = artist.Artist(self.config, artistName, sortName, artistId)

        try:
            self.asin = mb.GetResultData(musicbrainz.MBE_AlbumGetAmazonAsin)
        except musicbrainz.MusicBrainzError:
            self.asin = u''

        try:
            dummy, self.releaseType = mb.GetResultData(musicbrainz.MBE_AlbumGetAlbumType).split('Type')
        except musicbrainz.MusicBrainzError:
            self.releaseType = u''

        try:
            dummy, self.releaseStatus = mb.GetResultData(musicbrainz.MBE_AlbumGetAlbumStatus).split('Status')
        except musicbrainz.MusicBrainzError:
            self.releaseStatus = u''
    
        self.releaseCountry = u""
        self.releaseYear = 0
        self.releaseDate = u""
        try:
            numReleases = mb.GetResultInt(musicbrainz.MBE_AlbumGetNumReleaseDates)
            for i in xrange(1, numReleases + 1):
                mb.Select1(musicbrainz.MBS_SelectReleaseDate, i)
                releaseDate = mb.GetResultData(musicbrainz.MBE_ReleaseGetDate)
                releaseCountry = mb.GetResultData(musicbrainz.MBE_ReleaseGetCountry)

                releaseYear = int(releaseDate[0:4])
                if releaseYear < self.releaseYear or self.releaseYear == 0:
                    self.releaseYear = releaseYear
                    self.releaseCountry = releaseCountry
                    self.releaseDate = releaseDate

                mb.Select(musicbrainz.MBS_Back)
               
        except musicbrainz.MusicBrainzError:
            pass

        self.trackIds = {}
        self.duration = 0
        for i in xrange(1, numTracks + 1):
            name = mb.GetResultData1(musicbrainz.MBE_AlbumGetTrackName, i)
            duration = mb.GetResultInt1(musicbrainz.MBE_AlbumGetTrackDuration, i)
            trackId = mb.GetIDFromURL(mb.GetResultData1(musicbrainz.MBE_AlbumGetTrackId, i))

            self.duration += int(duration)

            try:
                tr = self.tracks[i-1]
                tr.setName(name)
                tr.setId(trackId)
            except IndexError:
                tr = track.Track(self.config, name, trackId)
                self.tracks.append(tr)
               
            tr.setNum(i)
            tr.setDuration(duration)
            tr.setAlbum(self)
            if self.isVA:
                artistName = mb.GetResultData1(musicbrainz.MBE_AlbumGetArtistName, i)
                artistId = mb.GetIDFromURL(mb.GetResultData1(musicbrainz.MBE_AlbumGetArtistId, i))
                sortName = mb.GetResultData1(musicbrainz.MBE_AlbumGetArtistSortName, i)
                tr.setArtist(artist.Artist(self.config, artistName, sortName, artistId))
            else:
                tr.setArtist(self.artist)
               
            tr.update()
            if tr.hasChanged() and (tr.getFileId() not in self.unsavedFiles):
                self.unsavedFiles.append(tr.getFileId())
               
            self.trackIds[tr.getId()] = i-1

        for i in xrange(numTracks + 1, len(self.tracks)):
            tr = self.tracks[i - 1]
            if tr.isLinked():
                fileId = tpfile.getFileId()
                del self.TPFileIdToTrackSeq[fileId]
                track.unlink()
                self.numLinked = self.numLinked - 1
            else:
                self.unmatchedFiles.remove(fileId)
            if fileId in self.unsavedFiles:
                self.unsavedFiles.remove(fileId)
               
        self.loaded = True

    def setServerAffinity(self, server, port):
        '''
        This sets the server that this album was loaded from. Future updates about
        this album should be pulled from the same server. If no affinity servers are
        set, the server in the config settings should be used.
        '''
        self.affServer = server
        self.affPort = port
  
    def getServerAffinity(self):
        '''
        Returns a tuple of the server and port that this album was loaded from. 
        See setServerAffinity()
        '''
        return (self.affServer, self.affPort)
