# ***** BEGIN LICENSE BLOCK *****
# Version: RCSL 1.0/RPSL 1.0/GPL 2.0
#
# Portions Copyright (c) 1995-2002 RealNetworks, Inc. All Rights Reserved.
# Portions Copyright (c) 2004 Robert Kaye. All Rights Reserved.
#
# The contents of this file, and the files included with this file, are
# subject to the current version of the RealNetworks Public Source License
# Version 1.0 (the "RPSL") available at
# http://www.helixcommunity.org/content/rpsl unless you have licensed
# the file under the RealNetworks Community Source License Version 1.0
# (the "RCSL") available at http://www.helixcommunity.org/content/rcsl,
# in which case the RCSL will apply. You may also obtain the license terms
# directly from RealNetworks.  You may not use this file except in
# compliance with the RPSL or, if you have a valid RCSL with RealNetworks
# applicable to this file, the RCSL.  Please see the applicable RPSL or
# RCSL for the rights, obligations and limitations governing use of the
# contents of the file.
#
# This file is part of the Helix DNA Technology. RealNetworks is the
# developer of the Original Code and owns the copyrights in the portions
# it created.
#
# This file, and the files included with this file, is distributed and made
# available on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
# EXPRESS OR IMPLIED, AND REALNETWORKS HEREBY DISCLAIMS ALL SUCH WARRANTIES,
# INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
#
# Technology Compatibility Kit Test Suite(s) Location:
#    http://www.helixcommunity.org/content/tck
#
# --------------------------------------------------------------------
#
# picard is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# picard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with picard; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Robert Kaye
#
#
# ***** END LICENSE BLOCK *****

import wx, copy
#import sys

# Here's a quick ready-reference as to where code needs to be added for each
# event:
# * Copy + paste by example in this file (wx.NewId(), an EVT* function, a class)
# * In tagger.py, create a handler function (def OnFoo)
# * In tagger.py, call the connector function EVT_FOO to connect the event to OnFoo

################################################################################

EVT_CLUSTER_ADDED_ID = wx.NewId()
EVT_CLUSTER_REMOVED_ID = wx.NewId()
EVT_CLUSTER_CHANGED_ID = wx.NewId()
EVT_CLUSTER_FILE_ADDED_ID = wx.NewId()
EVT_CLUSTER_FILE_REMOVED_ID = wx.NewId()

def EVT_CLUSTER_ADDED(win, func):
    win.Connect(-1, -1, EVT_CLUSTER_ADDED_ID, func)

def EVT_CLUSTER_REMOVED(win, func):
    win.Connect(-1, -1, EVT_CLUSTER_REMOVED_ID, func)

def EVT_CLUSTER_CHANGED(win, func):
    win.Connect(-1, -1, EVT_CLUSTER_CHANGED_ID, func)

class ClusterEvent(wx.PyEvent):
    """
    Triggered when a cluster is added, removed or changed (file added or
    removed)
    """

    def __init__(self, cluster, type):
        wx.PyEvent.__init__(self)
        self.cluster = cluster
	assert(type in (EVT_CLUSTER_ADDED_ID, EVT_CLUSTER_REMOVED_ID, EVT_CLUSTER_CHANGED_ID))
	self.SetEventType(type)

    def getCluster(self):
        return self.cluster

def EVT_CLUSTER_FILE_ADDED(win, func):
    win.Connect(-1, -1, EVT_CLUSTER_FILE_ADDED_ID, func)

def EVT_CLUSTER_FILE_REMOVED(win, func):
    win.Connect(-1, -1, EVT_CLUSTER_FILE_REMOVED_ID, func)

class ClusterFileEvent(wx.PyEvent):
    """
    Triggered when a file is added to, or removed from, a cluster
    """

    def __init__(self, cluster, fileId, type):
        wx.PyEvent.__init__(self)
        self.cluster = cluster
        self.fileId = fileId
	assert(type in (EVT_CLUSTER_FILE_ADDED_ID, EVT_CLUSTER_FILE_REMOVED_ID))
	self.SetEventType(type)

    def getCluster(self):
        return self.cluster

    def getFileId(self):
        return self.fileId

################################################################################
# From now on these events all use "anonymous" event IDs
################################################################################

id = wx.NewId()
def EVT_BROWSER_MSG(win, func, type=id): win.Connect(-1, -1, type, func)

class BrowserMessageEvent(wx.PyEvent):

    def __init__(self, action, args, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.action = action
        self.args = args

    def getAction(self):
        return self.action
   
    def getArgs(self):
        return self.args

################################################################################

class AlbumEvent(wx.PyEvent):
    """
    A base class for various album-related events
    """

    def __init__(self, mbid):
        wx.PyEvent.__init__(self)
        self.mbid = mbid

    def getId(self):
        return self.mbid

    def setId(self, mbid):
        self.mbid = mbid

########################################

"""
Triggered when an album's pending load has completed (which may or may
not have succeeded)
"""

id = wx.NewId()
def EVT_ALBUM_UPDATE(win, func, type=id): win.Connect(-1, -1, type, func)

class AlbumUpdateEvent(AlbumEvent):

    def __init__(self, mbid, type=id):
        AlbumEvent.__init__(self, mbid)
        self.SetEventType(type)

########################################

"""
Triggered when an album's track counts may have changed
"""

id = wx.NewId()
def EVT_ALBUM_TRACK_COUNTS_CHANGED(win, func, type=id): win.Connect(-1, -1, type, func)

class AlbumTrackCountsChangedEvent(AlbumEvent):

    def __init__(self, mbid, type=id):
        AlbumEvent.__init__(self, mbid)
        self.SetEventType(type)

########################################

"""
Triggered when an album has been removed from the album manager
"""

id = wx.NewId()
def EVT_ALBUM_REMOVED(win, func, type=id): win.Connect(-1, -1, type, func)

class AlbumRemovedEvent(AlbumEvent):

    def __init__(self, mbid, type=id):
        AlbumEvent.__init__(self, mbid)
        self.SetEventType(type)

########################################

"""
The user has requested than an album be removed
"""

id = wx.NewId()
def EVT_ALBUM_REMOVE(win, func, type=id): win.Connect(-1, -1, type, func)

class AlbumRemoveEvent(AlbumEvent):

    def __init__(self, mbid, type=id):
        AlbumEvent.__init__(self, mbid)
        self.SetEventType(type)

########################################

"""
Triggered when a file has been removed from an album
"""

id = wx.NewId()
def EVT_ALBUM_FILES_REMOVED(win, func, type=id): win.Connect(-1, -1, type, func)

class AlbumFilesRemovedEvent(AlbumEvent):

    def __init__(self, mbid, fileIdList, type=id):
        AlbumEvent.__init__(self, mbid)
        self.SetEventType(type)
        self.fileIdList = fileIdList

    def getFileList(self):
        return self.fileIdList

########################################

"""
The user has requested that an album's unmatched files be removed
"""

id = wx.NewId()
def EVT_ALBUM_REMOVE_UNMATCHED_FILES(win, func, type=id): win.Connect(-1, -1, type, func)

class AlbumRemoveUnmatchedFilesEvent(AlbumEvent):

    def __init__(self, mbid, type=id):
        AlbumEvent.__init__(self, mbid)
        self.SetEventType(type)

########################################

"""
The user has requested that an album's unmatched files all be moved back
to the main "unmatched files" set
"""

id = wx.NewId()
def EVT_MOVE_ALBUM_UNMATCHED_TO_UNMATCHED(win, func, type=id): win.Connect(-1, -1, type, func)

class MoveAlbumUnmatchedToUnmatchedEvent(AlbumEvent):

    def __init__(self, mbid, type=id):
        AlbumEvent.__init__(self, mbid)
        self.SetEventType(type)

################################################################################

class CoverArtEvent(wx.PyEvent):

    def __init__(self, asin):
        wx.PyEvent.__init__(self)
        self.asin = asin

    def getAsin(self):
        return self.asin
   
    def setAsin(self, asin):
        self.asin = asin

########################################

"""
Triggered when an album's cover art has been loaded, ready for display
"""

id = wx.NewId()
def EVT_COVERART_LOAD(win, func, type=id): win.Connect(-1, -1, type, func)

class CoverArtLoadEvent(CoverArtEvent):

    def __init__(self, asin, type=id):
        CoverArtEvent.__init__(self, asin)
        self.SetEventType(type)

########################################

"""
TODO describe this event
"""

id = wx.NewId()
def EVT_COVERART_SHOW(win, func, type=id): win.Connect(-1, -1, type, func)

class CoverArtShowEvent(CoverArtEvent):

    def __init__(self, asin, type=id):
        CoverArtEvent.__init__(self, asin)
        self.SetEventType(type)

################################################################################

"""
The user has requested that some file(s) be added to tunepimp
"""

id = wx.NewId()
def EVT_FILE_ADD(win, func, type=id): win.Connect(-1, -1, type, func)

class FileAddEvent(wx.PyEvent):

    def __init__(self, fileNames, albumId = "", albumSeq = "", type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.fileNames = copy.copy(fileNames)
        self.albumId = copy.copy(albumId)
        self.albumSeq = copy.copy(albumSeq)

    def getFileNames(self):
        return self.fileNames

    def getAlbumId(self):
        return self.albumId

    def getAlbumSeq(self):
        return self.albumSeq

################################################################################

"""
The user has requested that some directory/ies (the files therein)
be added to tunepimp
"""

id = wx.NewId()
def EVT_DIR_ADD(win, func, type=id): win.Connect(-1, -1, type, func)

class DirAddEvent(wx.PyEvent):

    def __init__(self, dirNames, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.dirNames = dirNames

    def getDirNames(self):
        return self.dirNames

################################################################################

"""
TODO describe this event
"""

id = wx.NewId()
def EVT_SHOW_ITEM_DETAILS(win, func, type=id): win.Connect(-1, -1, type, func)

class ShowItemDetailsEvent(wx.PyEvent):

    def __init__(self, item, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.item = item

    def getItem(self):
        return self.item

################################################################################

"""
TODO describe this event
"""

id = wx.NewId()
def EVT_SHOW_DIRECTORY(win, func, type=id): win.Connect(-1, -1, type, func)

class ShowDirectoryEvent(wx.PyEvent):

    def __init__(self, dir, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.dir = dir

    def getDir(self):
        return self.dir

################################################################################

class LinkEvent(wx.PyEvent):
    """
    A base class for events for linking and unlinking tracks / files
    """

    def __init__(self, trackId, fileId):
        wx.PyEvent.__init__(self)
        self.trackId = trackId
        self.fileId = fileId

    def getTrackId(self):
        return self.trackId

    def getFileId(self):
        return self.fileId

########################################

"""
The user has requested that a file be linked to a track
"""

id = wx.NewId()
def EVT_LINK_FILE(win, func, type=id): win.Connect(-1, -1, type, func)

class LinkFileEvent(LinkEvent):

    def __init__(self, trackId, fileId, type=id):
        LinkEvent.__init__(self, trackId, fileId)
        self.SetEventType(type)

################################################################################

"""
Some files are ready to be saved.  Triggered indirectly by the "save" button.
"""

id = wx.NewId()
def EVT_SAVE_FILE(win, func, type=id): win.Connect(-1, -1, type, func)

class SaveFileEvent(wx.PyEvent):

    def __init__(self, fileIdList, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.fileIdList = fileIdList

    def getFileIdList(self):
        return self.fileIdList

################################################################################

class FileEvent(wx.PyEvent):
    """
    A base class for various tunepimp file -related events
    """

    def __init__(self, fileId):
        wx.PyEvent.__init__(self)
        self.fileId = fileId

    def getFileId(self):
        return self.fileId

########################################

"""
TODO describe this event
"""

id = wx.NewId()
def EVT_UPDATE_FILE(win, func, type=id): win.Connect(-1, -1, type, func)

class UpdateFileEvent(FileEvent):

    def __init__(self, fileId, type=id):
        FileEvent.__init__(self, fileId)
        self.SetEventType(type)

########################################

"""
Analyze the file again after the user corrected some error
"""

id = wx.NewId()
def EVT_ANALYZE_FILE(win, func, type=id): win.Connect(-1, -1, type, func)

class AnalyzeFileEvent(wx.PyEvent):

    def __init__(self, fileIdList, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.fileIdList = fileIdList

    def getFileIdList(self):
        return copy.copy(self.fileIdList)

########################################

"""
The user cleared an error from a track and wants to attempt saving it again
"""

id = wx.NewId()
def EVT_CLEAR_ERROR(win, func, type=id): win.Connect(-1, -1, type, func)

class ClearErrorEvent(FileEvent):

    def __init__(self, fileId, type=id):
        FileEvent.__init__(self, fileId)
        self.SetEventType(type)

########################################

"""
The user wants to force saving this file, even if libtp doesn't think anything has changed.
"""

id = wx.NewId()
def EVT_FORCE_SAVE(win, func, type=id): win.Connect(-1, -1, type, func)

class ForceSaveEvent(FileEvent):

    def __init__(self, fileId, type=id):
        FileEvent.__init__(self, fileId)
        self.SetEventType(type)

########################################

"""
The user wants to reload the album from the server since the server data may have changed.
"""

id = wx.NewId()
def EVT_RELOAD_ALBUM(win, func, type=id): win.Connect(-1, -1, type, func)

class ReloadAlbumEvent(AlbumEvent):

    def __init__(self, albumId, type=id):
        AlbumEvent.__init__(self, albumId)
        self.SetEventType(type)

########################################

"""
The user wants to generate a cuesheet.
"""

id = wx.NewId()
def EVT_GENERATE_CUESHEET(win, func, type=id): win.Connect(-1, -1, type, func)

class GenerateCuesheetEvent(AlbumEvent):

    def __init__(self, albumId, type=id):
        AlbumEvent.__init__(self, albumId)
        self.SetEventType(type)

########################################

"""
The user wants to generate a M3U playlist.
"""

id = wx.NewId()
def EVT_GENERATE_PLAYLIST(win, func, type=id): win.Connect(-1, -1, type, func)

class GeneratePlaylistEvent(AlbumEvent):

    def __init__(self, albumId, type=id):
        AlbumEvent.__init__(self, albumId)
        self.SetEventType(type)

################################################################################

"""
The user has requested that some files be moved to "unmatched files"
"""

id = wx.NewId()
def EVT_MOVE_FILES_TO_UNMATCHED(win, func, type=id): win.Connect(-1, -1, type, func)

class MoveFilesToUnmatchedEvent(wx.PyEvent):

    def __init__(self, fileIdList, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.fileIdList = fileIdList

    def getFileIdList(self):
        return copy.copy(self.fileIdList)

################################################################################

"""
The user has requested that some files (given by filename) be matched to
an album
"""

id = wx.NewId()
def EVT_MATCH_FILES_TO_ALBUM(win, func, type=id): win.Connect(-1, -1, type, func)

class MatchFilesToAlbumEvent(wx.PyEvent):

    def __init__(self, fileList, albumId, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.fileList = fileList
        self.albumId = albumId

    def getFileList(self):
        return self.fileList

    def getAlbumId(self):
        return self.albumId

################################################################################

"""
The user has requested that some files be moved onto some album
OR
file matching has been triggered because an album has just finished loading
"""

id = wx.NewId()
def EVT_MATCH_FILEIDS_TO_ALBUM(win, func, type=id): win.Connect(-1, -1, type, func)

class MatchFileIdsToAlbumEvent(wx.PyEvent):

    def __init__(self, fileIdList, albumId, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.fileIdList = fileIdList
        self.albumId = albumId

    def getFileIdList(self):
        return self.fileIdList

    def getAlbumId(self):
        return self.albumId

################################################################################

"""
The user has matched (dragged) a directory to an album
"""

id = wx.NewId()
def EVT_MATCH_DIR_TO_ALBUM(win, func, type=id): win.Connect(-1, -1, type, func)

class MatchDirToAlbumEvent(wx.PyEvent):

    def __init__(self, dirName, albumId, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.dirName = dirName
        self.albumId = albumId

    def getDirName(self):
        return self.dirName

    def getAlbumId(self):
        return self.albumId

################################################################################

"""
The user has requested that a file be moved into some cluster
"""

id = wx.NewId()
def EVT_ADD_FILE_TO_CLUSTER(win, func, type=id): win.Connect(-1, -1, type, func)

class AddFileToClusterEvent(wx.PyEvent):

    def __init__(self, fileId, clusterId, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.fileId = fileId
        self.clusterId = clusterId

    def getFileId(self):
        return self.fileId

    def getClusterId(self):
        return self.clusterId

################################################################################

"""
The user has requested that a cluster be removed
"""

id = wx.NewId()
def EVT_REMOVE_CLUSTER_ALBUM(win, func, type=id): win.Connect(-1, -1, type, func)

class RemoveClusterAlbumEvent(wx.PyEvent):

    def __init__(self, clusterId, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.clusterId = clusterId

    def getClusterId(self):
        return self.clusterId

################################################################################

"""
The user has requested that unmatched files be clustered
"""

id = wx.NewId()
def EVT_CLUSTER_UNMATCHED_FILES(win, func, type=id): win.Connect(-1, -1, type, func)

class ClusterUnmatchedFilesEvent(wx.PyEvent):

    def __init__(self, threshold, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.threshold = threshold

    def getThreshold(self):
        return self.threshold

################################################################################

"""
The user has requested that an album (???) be moved to "unmatched files"
"""

id = wx.NewId()
def EVT_MOVE_CLUSTERS_TO_UNMATCHED(win, func, type=id): win.Connect(-1, -1, type, func)

class MoveClustersToUnmatchedEvent(wx.PyEvent):

    def __init__(self, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)

################################################################################

"""
The tunepimp settings have changed
"""

id = wx.NewId()
def EVT_SETTINGS_CHANGED(win, func, type=id): win.Connect(-1, -1, type, func)

class SettingsChangedEvent(wx.PyEvent):

    def __init__(self, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)

################################################################################

"""
The user has requested that some file be removed from tunepimp
"""

id = wx.NewId()
def EVT_REMOVE_TPFILE_COMMAND(win, func, type=id): win.Connect(-1, -1, type, func)

class RemoveTPFileCommand(wx.PyEvent):

    def __init__(self, fileId, type=id):
        wx.PyEvent.__init__(self)
        self.SetEventType(type)
        self.fileId = fileId

    def getFileId(self):
	return self.fileId

################################################################################

"""
Triggered when a file has been added to an album (either linked to a
specific track, or not ('unmatched')
"""

id = wx.NewId()
def EVT_ALBUM_FILES_ADDED(win, func, type=id): win.Connect(-1, -1, type, func)

class AlbumFilesAddedEvent(AlbumEvent):

    def __init__(self, mbid, fileIdList, type=id):
        AlbumEvent.__init__(self, mbid)
        self.SetEventType(type)
        self.fileIdList = fileIdList

    def getFileList(self):
        return self.fileIdList

################################################################################

"""
Triggered when an album has been added to the album manager
"""

id = wx.NewId()
def EVT_ALBUM_ADD(win, func, type=id): win.Connect(-1, -1, type, func)

class AlbumAddEvent(AlbumEvent):

    def __init__(self, mbid, type=id):
        AlbumEvent.__init__(self, mbid)
        self.SetEventType(type)

################################################################################

del id
