#!/usr/bin/env python

from distutils.core import setup
import shutil
import glob
import wx
import tagger

includes = ["encodings", "encodings.latin_1", "encodings.UTF-8"]

# NSIS installer

from py2exe.build_exe import py2exe

class bdist_nsis(py2exe):
    def run(self):
    
        py2exe.run(self)
        
        print "*** creating the NSIS setup script ***"
        
        pathname = "installer/picard-setup.nsi"
        
        file = open(pathname + ".in", "rt")
        script = file.read()
        file.close()
        
        script = script % dict(name = tagger.APPNAME, version = tagger.APPVERSION)
        
        file = open(pathname, "wt")
        file.write(script);
        file.close()
        
        print "*** compiling the NSIS setup script ***"
        
        try:
            import ctypes
        except ImportError:
            try:
                import win32api
            except ImportError:
                import os
                os.startfile(pathname)
            else:
                print "Ok, using win32api."
                win32api.ShellExecute(0, "compile", pathname, None, None, 0)
        else:
            print "Cool, you have ctypes installed."
            res = ctypes.windll.shell32.ShellExecuteA(0, "compile", pathname, None, None, 0)
            if res < 32:
                raise RuntimeError, "ShellExecute failed, error %d" % res

# manifest for Windows XP

manifest = '''
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<assembly xmlns="urn:schemas-microsoft-com:asm.v1" manifestVersion="1.0">
<assemblyIdentity
    version="0.0.0.0"
    processorArchitecture="x86"
    name="%(name)s"
    type="win32"
/>
<description>%(name)s</description>
<dependency>
    <dependentAssembly>
        <assemblyIdentity
            type="win32"
            name="Microsoft.Windows.Common-Controls"
            version="6.0.0.0"
            processorArchitecture="X86"
            publicKeyToken="6595b64144ccf1df"
            language="*"
        />
    </dependentAssembly>
</dependency>
</assembly>
''' % dict(name = tagger.APPNAME)

RT_MANIFEST = 24

# data files

data_files = [
    ("", glob.glob("*.dll")),
    ("rc", ["rc\\picard.ico"]),
    ("plugins", glob.glob("plugins\\*.tpp")),
    ]
    
for loc in glob.glob("locale/*"):
    data_files.append(("%s\\LC_MESSAGES" % loc, ["%s\\LC_MESSAGES\\picard.mo" % loc]))
    
# setup
    
setup(
    windows = [{
        "script"          : "tagger.py",
        "icon_resources"  : [(1, "rc\\picard.ico")],
        "other_resources" : [(RT_MANIFEST, 1, manifest)]
        }],
    data_files = data_files,
    options = {
        "py2exe"    : { "packages" : [ "encodings" ], "optimize" : 2}, 
        "win32file" : { "packages" : [ "win32api"  ]},
        "win32api"  : { "packages" : [ "win32api"  ]},
        "win32con"  : { "packages" : [ "win32api"  ]}
        },
    cmdclass = {"bdist_nsis": bdist_nsis}
    )
