#!/usr/bin/env python
# ***** BEGIN LICENSE BLOCK *****
# Version: RCSL 1.0/RPSL 1.0/GPL 2.0
#
# Portions Copyright (c) 1995-2002 RealNetworks, Inc. All Rights Reserved.
# Portions Copyright (c) 2004 Robert Kaye. All Rights Reserved.
#
# The contents of this file, and the files included with this file, are
# subject to the current version of the RealNetworks Public Source License
# Version 1.0 (the "RPSL") available at
# http://www.helixcommunity.org/content/rpsl unless you have licensed
# the file under the RealNetworks Community Source License Version 1.0
# (the "RCSL") available at http://www.helixcommunity.org/content/rcsl,
# in which case the RCSL will apply. You may also obtain the license terms
# directly from RealNetworks.  You may not use this file except in
# compliance with the RPSL or, if you have a valid RCSL with RealNetworks
# applicable to this file, the RCSL.  Please see the applicable RPSL or
# RCSL for the rights, obligations and limitations governing use of the
# contents of the file.
#
# This file is part of the Helix DNA Technology. RealNetworks is the
# developer of the Original Code and owns the copyrights in the portions
# it created.
#
# This file, and the files included with this file, is distributed and made
# available on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
# EXPRESS OR IMPLIED, AND REALNETWORKS HEREBY DISCLAIMS ALL SUCH WARRANTIES,
# INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
#
# Technology Compatibility Kit Test Suite(s) Location:
#    http://www.helixcommunity.org/content/tck
#
# --------------------------------------------------------------------
#
# picard is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# picard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with picard; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Contributor(s):
#   Robert Kaye
#
#
# ***** END LICENSE BLOCK *****

import wx, os, sys, copy
import pydirctrl, albumpanel
from lib import events, wpath

class TaggerDirCtrl(pydirctrl.PyDirCtrl):
    
    selections = []
    menuRefreshId = wx.NewId()

    def __init__(self, parent, frame, albumPanel, id, config, dirCache):
        pydirctrl.PyDirCtrl.__init__(self, parent, id, initialDir=config.persistDirControlPath, 
                                     cache=dirCache, config=config)
        self.frame = frame
        self.config = config
        self.id = id
        self.albumPanel = albumPanel

        wx.EVT_TREE_BEGIN_DRAG(self, self.id, self.OnBeginDrag)
        wx.EVT_TREE_SEL_CHANGED(self, self.id, self.OnSelChanged)
        wx.EVT_RIGHT_DOWN(self, self.OnRightDown)
        wx.EVT_MENU(self, self.menuRefreshId, self.OnRefresh)

        wx.PostEvent(self.frame, events.ShowDirectoryEvent(config.persistDirControlPath))
        wx.WakeUpIdle()

    def OnSelChanged(self, event):

        paths = self.getSelectedPaths()
        if len(paths) == 0:
            return

        if len(paths) == 1:
            self.config.persistDirControlPath = paths[0]
            wx.PostEvent(self.frame, events.ShowDirectoryEvent(copy.copy(paths[0])))
        else:
            wx.PostEvent(self.frame, events.ShowDirectoryEvent(""))
        wx.WakeUpIdle()

    def OnBeginDrag(self, event): 

        selections = self.getSelectedPaths()
        if len(selections) > 1:
            dropText = u"dirs:"
            for path in selections:
               dropText = dropText + path + u"\n"
            dropText = dropText[:len(dropText)-1]

            tdo = wx.PyTextDataObject(dropText)
        else:
            selPath = selections[0]
            if wpath.wpath(self.config).isdir(selPath):
                tdo = wx.PyTextDataObject(u"dir:" + selPath)
            else:
                event.Veto()
                return

        event.Allow()
        tds = albumpanel.TaggerDropSource(self, self.albumPanel) 
        tds.SetData(tdo) 
        tds.DoDragDrop(True) 
        event.Skip()

    def OnRightDown(self, event):
        item, flags = self.HitTest(wx.Point(event.m_x, event.m_y))
        if item:
            if not self.IsSelected(item):
                self.UnselectAll()
                self.SelectItem(item)
            popupMenu = wx.Menu()
            popupMenu.Append(self.menuRefreshId, u"Refresh")
            self.PopupMenuXY(popupMenu, event.m_x, event.m_y)

    def OnRefresh(self, event):
        sels = self.GetSelections()
        for sel in sels:
            path = self.GetPyData(sel)
            if path:
                self.refreshPath(path) 
